#
# This file is part of python-rhev. python-rhev is free software that is
# made available under the MIT license. Consult the file "LICENSE" that
# is distributed together with this file for the exact licensing terms.
#
# python-rhev is copyright (c) 2010 by the python-rhev authors. See the
# file "AUTHORS" for a complete overview.

from rhev import *
from rhev.test import util
from rhev.test.base import BaseTest
from rhev.test.loader import depends
from nose import SkipTest
from nose.tools import assert_raises


class TestStorage(BaseTest):

    def _prepare(self, name):
        type = self.get_config('type', name)
        if type is None:
            raise SkipTest, 'Missing parameter "type" in [%s]' % name
        if type == 'nfs':
            params = ('address', 'path')
        elif type == 'iscsi':
            params = ('address', 'port', 'target', 'username', 'password')
        elif type == 'fcp':
            params = ()
        else:
            raise SkipTest, 'Uknown storage type %s in [%s]' % (type, name)
        values = { 'type': type }
        for param in params:
            value = self.get_config(param, name)
            if value is None:
                raise SkipTest, 'Missing parameter "%s" in [%s]' % (param, name)
            values[param] = value
        if values['type'] in ('iscsi', 'fcp'):
            luns = self.get_config('luns', name)
            if luns is None:
                raise SkipTest, 'Specify "luns" in [%s]' % name
            values['luns'] = luns.split(',')
        hosts = self.get_config('hosts').split()
        if len(hosts) == 0:
            raise SkipTest, 'No hosts configured.'
        host = self.api.get(schema.Host, name=hosts[0])
        if host is None:
            raise SkipTest, 'Host not found: %s' % hosts[0]
        values['host'] = host
        cpu = self.get_config('cpu', host.name)
        if cpu is None:
            raise SkipTest, 'Missing parameter "cpu" in [%s]' % host.name
        values['cpu'] = cpu
        for key in values:
            setattr(self.store, key, values[key])
        datacenter = schema.new(schema.DataCenter)
        datacenter.name = util.random_name('dc')
        datacenter.storage_type = self.store.type
        datacenter.version = self.get_version()
        datacenter = self.api.create(datacenter)
        assert isinstance(datacenter, schema.DataCenter)
        assert datacenter.id is not None
        cluster = schema.new(schema.Cluster)
        cluster.name = util.random_name('cluster')
        cluster.cpu = schema.new(schema.CPU)
        cluster.cpu.id = self.store.cpu
        cluster.data_center = schema.ref(datacenter)
        cluster.version = datacenter.version
        cluster = self.api.create(cluster)
        assert isinstance(cluster, schema.Cluster)
        assert cluster.id is not None
        self.store.datacenter = datacenter
        self.store.cluster = cluster
        if host.status.state != 'maintenance':
            assert self.retry_action(host, 'deactivate')
            assert self.wait_for_status(host, 'maintenance')
        self.store.clusterid = host.cluster.id
        host.cluster = schema.ref(cluster)
        self.api.update(host)
        assert host.cluster.id == cluster.id
        self.api.action(host, 'activate')
        assert self.wait_for_status(host, 'up')

    @depends(_prepare)
    def _test_iscsi_discover(self):
        host = self.store.host
        target = self.store.target
        action = schema.new(schema.Action)
        action.iscsi = schema.new(schema.IscsiDetails)
        action.iscsi.address = self.store.address
        action.iscsi.port = self.store.port
        result = self.api.action(host, 'iscsidiscover', action)
        assert result.status.state == 'complete'
        if target not in result.iscsi_target:
            raise SkipTest, 'iscsi target not found: %s' % target

    @depends(_test_iscsi_discover)
    def _test_iscsi_login(self):
        host = self.store.host
        action = schema.new(schema.Action)
        action.iscsi = schema.new(schema.IscsiDetails)
        action.iscsi.address = self.store.address
        action.iscsi.port = self.store.port
        action.iscsi.target = self.store.target
        if self.store.username:
            action.iscsi.username = self.store.username
        if self.store.password:
            action.iscsi.password = self.store.password
        result = self.api.action(host, 'iscsilogin', action)
        assert result.status.state == 'complete'

    def _find_storage(self, storage, type, id):
        for stor in storage:
            # BZ 738060
            #if stor.type != type:
            #    continue
            if stor.logical_unit is None:
                continue
            if stor.logical_unit[0].id == id:
                return stor.logical_unit[0]
        return None

    @depends(_test_iscsi_login, _prepare)
    def _test_host_storage(self):
        host = self.store.host
        type = self.store.type
        storage = self.api.getall(schema.Storage, base=host)
        luns = []
        for lun in self.store.luns:
            storage = self._find_storage(storage, type, lun)
            assert storage is not None
            luns.append(storage)
        self.store.luns = luns

    @depends(_test_host_storage)
    def _test_host_storage_attributes(self):
        host = self.store.host
        storage = self.api.getall(schema.HostStorage, base=host)
        assert isinstance(storage, schema.HostStorage)
        for stor in storage:
            # BZ #741668
            if stor.volume_group:
                continue
            assert isinstance(stor, schema.Storage)
            assert util.is_str(stor.id)
            assert util.is_str_href(stor.href)
            # BZ #738060
            #assert stor.type in ('iscsi', 'fcp')
            assert stor.host is not None
            assert util.is_str_uuid(stor.host.id)
            assert util.is_str_href(stor.host.href)
            lun = stor.logical_unit[0]
            assert lun.id == stor.id
            assert util.is_str(lun.portal)
            assert util.is_str_ip(lun.address)
            assert util.is_int(lun.port)
            assert util.is_str(lun.username)
            assert util.is_str(lun.target)
            assert util.is_str(lun.vendor_id)
            assert util.is_str(lun.product_id)
            assert util.is_int(lun.lun_mapping)
            assert util.is_int(lun.size)
            assert util.is_int(lun.paths)

    @depends(_prepare)
    def _test_create_nfs_data(self):
        host = self.store.host
        datacenter = self.store.datacenter
        domain = schema.new(schema.StorageDomain)
        domain.name = util.random_name('sd')
        domain.type = 'data'
        domain.storage = schema.new(schema.Storage)
        domain.storage.type = 'nfs'
        domain.storage.address = self.store.address
        domain.storage.path = self.store.path
        domain.host = schema.ref(host)
        domain2 = self.api.create(domain)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id is not None
        domain = self.api.get(schema.StorageDomain, id=domain2.id)
        assert isinstance(domain, schema.StorageDomain)
        assert domain.id == domain2.id
        assert domain.storage.type == 'nfs'
        assert domain.storage.address == self.store.address
        assert domain.storage.path == self.store.path
        self.store.storagedomain = domain
        self.store.attachdomain = None

    @depends(_test_host_storage)
    def _test_create_block_data(self):
        host = self.store.host
        datacenter = self.store.datacenter
        domain = schema.new(schema.StorageDomain)
        domain.name = util.random_name('sd')
        domain.type = 'data'
        domain.storage = schema.new(schema.Storage)
        domain.storage.type = self.store.type
        if self.store.luns:
            luns = [schema.ref(lun) for lun in self.store.luns ]
            domain.storage.logical_unit = luns
        elif self.store.volume_group:
            domain.storage.volume_group = [schema.ref(self.store.volume_group)]
        domain.host = schema.ref(host)
        domain2 = self.api.create(domain)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id is not None
        domain = self.api.get(schema.StorageDomain, id=domain2.id)
        assert isinstance(domain, schema.StorageDomain)
        assert domain.id == domain2.id
        assert domain.href == domain2.href
        assert domain.storage.type == self.store.type
        self.store.storagedomain = domain
        self.store.attachdomain = None

    @depends(_test_create_nfs_data, _test_create_block_data)
    def _test_create(self):
        """Dummy test for dependency tracking."""

    @depends(_test_create)
    def _test_get(self):
        domain = self.store.storagedomain
        domain2 = self.api.get(schema.StorageDomain, id=domain.id)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id == domain.id

    @depends(_test_create)
    def _test_reload(self):
        domain = self.store.storagedomain
        domain2 = self.api.reload(domain)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id == domain.id

    @depends(_test_create)
    def _test_getall(self):
        domain = self.store.storagedomain
        domains = self.api.getall(schema.StorageDomain)
        assert isinstance(domains, schema.StorageDomains)
        assert len(domains) > 0
        assert util.contains_id(domains, domain.id)

    @depends(_test_create)
    def _test_search(self):
        domain = self.store.storagedomain
        domains = self.api.getall(schema.StorageDomain, name=domain.name)
        assert isinstance(domains, schema.StorageDomains)
        assert len(domains) == 1
        assert domains[0].id == domain.id
        domains = self.api.getall(schema.StorageDomain, search='name=%s' % domain.name)
        assert isinstance(domains, schema.StorageDomains)
        assert len(domains) == 1
        assert domains[0].id == domain.id
    
    @depends(_test_create)
    def _test_attributes(self):
        domain = self.store.storagedomain
        assert util.is_str(domain.name)
        assert domain.type in ('data', 'iso', 'export')
        assert domain.storage is not None
        assert domain.storage.type in ('nfs', 'iscsi', 'fcp')
        if domain.storage.type == 'nfs':
            assert util.is_str_host(domain.storage.address)
            assert util.is_str_path(domain.storage.path)
        assert util.is_int(domain.available)
        assert util.is_int(domain.used)
        assert util.is_int(domain.committed)

    @depends(_test_create)
    def _test_attach(self):
        domain = self.store.storagedomain
        datacenter = self.store.datacenter
        domref = schema.ref(domain)
        domain2 = self.api.create(domref, base=datacenter)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id == domain.id
        assert domain2.href != domain.href
        assert self.wait_for_status(datacenter, 'up')
        self.store.attachdomain = domain2

    @depends(_test_attach)
    def _test_get_attached(self):
        datacenter = self.store.datacenter
        domain = self.store.attachdomain
        domain2 = self.api.get(schema.StorageDomain, base=datacenter, id=domain.id)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id == domain.id

    @depends(_test_attach)
    def _test_reload_attached(self):
        domain = self.store.attachdomain
        domain2 = self.api.reload(domain)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id == domain.id

    @depends(_test_attach)
    def _test_getall_attached(self):
        datacenter = self.store.datacenter
        domain = self.store.attachdomain
        domains = self.api.getall(schema.StorageDomain, base=datacenter)
        assert isinstance(domains, schema.StorageDomains)
        assert len(domains) > 0
        assert util.contains_id(domains, domain.id)

    @depends(_test_attach)
    def _test_attributes_attached(self):
        domain = self.store.attachdomain
        assert util.is_str(domain.name)
        assert domain.type in ('data', 'iso', 'export')
        assert domain.status.state in ('active', 'locked', 'inactive')
        assert domain.storage is not None
        assert domain.storage.type in ('nfs', 'iscsi', 'fcp')
        if domain.storage.type == 'nfs':
            assert util.is_str_host(domain.storage.address)
            assert util.is_str_path(domain.storage.path)
        assert util.is_int(domain.available)
        assert util.is_int(domain.used)
        assert util.is_int(domain.committed)

    @depends(_test_attach)
    def _test_update(self):
        domain = self.store.storagedomain
        domain.name = util.random_name('sd')
        domain.type = None
        domain2 = self.api.update(domain)
        assert domain2.id == domain.id
        assert domain2.name == domain.name

    @depends(_test_attach)
    def _test_create_iso(self, name):
        host = self.store.host
        address = self.get_config('address', name)
        if address is None:
            raise SkipTest, 'Parameter "address" not set in [%s]' % name
        path = self.get_config('path', name)
        if path is None:
            raise SkipTest, 'Parameter "path" not set in [%s]' % name
        domain = schema.new(schema.StorageDomain)
        domain.name = util.random_name('iso')
        domain.type = 'iso'
        domain.storage = schema.new(schema.Storage)
        domain.storage.type = 'nfs'
        domain.storage.address = address
        domain.storage.path = path
        domain.host = schema.ref(host)
        domain2 = self.api.create(domain)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id is not None
        self.store.isodomain = domain2

    @depends(_test_create_iso)
    def _test_attach_iso(self):
        datacenter = self.store.datacenter
        domain = self.store.isodomain
        domain2 = self.api.create(schema.ref(domain), base=datacenter)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id == domain.id
        assert domain2.href != domain.href
        self.wait_for_status(domain2, 'inactive')
        self.store.isoattach = domain2

    @depends(_test_attach_iso)
    def _test_activate_iso(self):
        domain = self.store.isoattach
        action = self.api.action(domain, 'activate')
        assert action.status.state == 'complete'
        assert self.wait_for_status(domain, 'active')

    @depends(_test_activate_iso)
    def _test_deactivate_iso(self):
        domain = self.store.isoattach
        action = self.api.action(domain, 'deactivate')
        assert action.status.state == 'complete'
        assert self.wait_for_status(domain, 'inactive')

    @depends(_test_deactivate_iso)
    def _test_detach_iso(self):
        datacenter = self.store.datacenter
        domain = self.store.isoattach
        self.api.delete(domain)
        domains = self.api.getall(schema.StorageDomain, base=datacenter)
        assert not util.contains_id(domains,  domain.id)
        domains = self.api.getall(schema.StorageDomain)
        assert util.contains_id(domains, domain.id)

    @depends(_test_detach_iso)
    def _test_destroy_iso(self):
        host = self.store.host
        domain = self.store.isodomain
        params = schema.new(schema.StorageDomain)
        params.host = schema.ref(host)
        params.format = False
        self.api.delete(domain, data=params)
        domains = self.api.getall(schema.StorageDomain)
        assert not util.contains_id(domains, domain.id)

    @depends(_test_destroy_iso)
    def _test_import_iso(self):
        host = self.store.host
        domain = self.store.isodomain
        domain2 = schema.new(schema.StorageDomain)
        domain2.name = None  # This identifies the creation as an import
        domain2.type = 'iso'
        domain2.storage = schema.new(schema.Storage)
        domain2.storage.type = 'nfs'
        domain2.storage.address = domain.storage.address
        domain2.storage.path = domain.storage.path
        domain2.host = schema.ref(host)
        domain2 = self.api.create(domain2)
        assert isinstance(domain2, schema.StorageDomain)
        assert domain2.id is not None
        assert domain2.id == domain.id
        assert domain2.name == domain.name

    @depends(_test_create_iso)
    def _test_delete_iso(self):
        host = self.store.host
        domain = self.store.isodomain
        params = schema.new(schema.StorageDomain)
        params.host = schema.ref(host)
        params.format = True
        self.api.delete(domain, data=params)
        domains = self.api.getall(schema.StorageDomain)
        assert not util.contains_id(domains, domain.id)

    @depends(_test_attach)
    def _test_deactivate(self):
        domain = self.store.attachdomain
        datacenter = self.store.datacenter
        action = self.api.action(domain, 'deactivate')
        assert action.status.state == 'complete'
        assert self.wait_for_status(domain, 'inactive')
        assert self.wait_for_status(datacenter, 'maintenance')

    @depends(_test_deactivate)
    def _test_activate(self):
        domain = self.store.attachdomain
        datacenter = self.store.datacenter
        action = self.api.action(domain, 'activate')
        assert action.status.state == 'complete'
        assert self.wait_for_status(domain, 'active')
        assert self.wait_for_status(datacenter, 'up')

    @depends(_test_create)
    def _test_delete(self):
        # We'd like to detach the storage domain but that's not possible as
        # this is the only data domain left. Therefore, delete the entire DC.
        domain = self.store.storagedomain
        atdomain = self.store.attachdomain
        cluster = self.store.cluster
        datacenter = self.store.datacenter
        host = self.store.host
        self.api.action(atdomain, 'deactivate')
        atdomain = self.api.reload(atdomain)
        assert atdomain.status.state == 'inactive'
        self.api.delete(datacenter)
        datacenter = self.api.get(schema.DataCenter, name=datacenter.name)
        assert datacenter is None
        params = schema.new(schema.StorageDomain)
        params.host = schema.ref(self.store.host)
        params.format = True
        self.api.delete(domain, data=params)
        # Loading by name because by ID -> 500 Internal Server Error
        domain = self.api.get(schema.StorageDomain, name=domain.name)
        assert domain is None
        assert self.retry_action(host, 'deactivate')
        assert self.wait_for_status(host, 'maintenance')
        host = schema.href(host)
        host.cluster = schema.new(schema.Cluster)
        host.cluster.id = self.store.clusterid
        host = self.api.update(host)
        assert host.cluster.id == self.store.clusterid
        action = self.api.action(host, 'activate')
        assert action.status.state == 'complete'
        assert self.wait_for_status(host, 'up')
        self.api.delete(cluster)
        cluster = self.api.get(schema.Cluster, name=cluster.name)
        assert cluster is None

    def test_storage_domain(self):
        domains = self.get_config('storage')
        domains = domains.split()
        for domain in domains:
            # Create the data domain
            yield self._prepare, domain
            type = self.store.type
            if type == 'iscsi':
                yield self._test_iscsi_discover
                yield self._test_iscsi_login
                yield self._test_host_storage
                yield self._test_host_storage_attributes
                yield self._test_create_block_data
            elif type == 'fcp':
                yield self._test_host_storage
                yield self._test_host_storage_attributes
                yield self._test_create_block_data
            elif type == 'nfs':
                yield self._test_create_nfs_data
            yield self._test_create
            yield self._test_get
            yield self._test_reload
            yield self._test_getall
            yield self._test_search
            yield self._test_attributes
            yield self._test_attach
            yield self._test_get_attached
            yield self._test_reload_attached
            yield self._test_getall_attached
            yield self._test_attributes_attached
            # We can only update an attached and activated SD
            yield self._test_update 
            yield self._test_deactivate
            yield self._test_activate
            iso = self.get_config('iso', domain)
            if iso is not None:
                yield self._test_create_iso, iso
                yield self._test_attach_iso
                yield self._test_activate_iso
                yield self._test_deactivate_iso
                yield self._test_detach_iso
                yield self._test_destroy_iso
                yield self._test_import_iso
                yield self._test_delete_iso
            # TBD: export domains
            yield self._test_delete

    def test_prepare_nonexistent(self):
        domain = schema.new(schema.StorageDomain)
        domain.id = 'foo'
        domain.href = '%s/storagedomains/foo' % self.api.entrypoint
        self.store.storagedomain = domain

    @depends(test_prepare_nonexistent)
    def test_get_nonexistent(self):
        domain = self.store.storagedomain
        domain = self.api.get(schema.StorageDomain, id=domain.id)
        assert domain is None

    @depends(test_prepare_nonexistent)
    def test_reload_nonexistent(self):
        domain = self.store.storagedomain
        assert_raises(NotFound, self.api.reload, domain)

    @depends(test_prepare_nonexistent)
    def test_update_nonexistent(self):
        domain = self.store.storagedomain
        assert_raises(NotFound, self.api.update, domain)

    @depends(test_prepare_nonexistent)
    def test_delete_nonexistent(self):
        domain = self.store.storagedomain
        assert_raises(NotFound, self.api.delete, domain)

    @depends(_test_create)
    def _test_has_files(self):
        dc = self.store.dc
        files = self.api.getall(schema.Files, base=dc)
        assert isinstance(files, schema.Files)
        for file in files:
            assert isinstance(file, schema.File)

    @depends(_test_has_files)
    def _test_file_attributes(self):
        dc = self.store.dc
        files = self.api.getall(schema.Files, base=dc)
        for file in files:
            assert util.is_str(file.id) and len(file.id) > 0
            assert util.is_str_href(file.href) and file.href.endswith(file.id)
            assert util.is_str(file.name)
            assert file.type in ('iso', 'vfd')
            assert isinstance(file.data_center, schema.DataCenter)
            assert file.data_center.id == dc.id
