#
# This file is part of python-rhev. python-rhev is free software that is
# made available under the MIT license. Consult the file "LICENSE" that
# is distributed together with this file for the exact licensing terms.
#
# python-rhev is copyright (c) 2010-2011 by the python-rhev authors. See
# the file "AUTHORS" for a complete overview.


import sys
import os
import logging
import traceback

from optparse import OptionParser
from urlparse import urlparse

import rhev
import psycopg2

parser = None
opts = None
args = None


def setup_logging(debug):
    """Set up logging."""
    if debug:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logger = logging.getLogger()
    handler = logging.StreamHandler(sys.stdout)
    format = '%(levelname)s: %(message)s'
    formatter = logging.Formatter(format)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(level)


def create_parser(usage=None, description=None):
    """Create a new option parser."""
    global parser
    parser = OptionParser(usage=usage, description=description)
    parser.add_option('-U', '--url',
                      help='the API entry point (default: $RHEV_URL)')
    parser.add_option('-u', '--username', dest='user',
                      help='the API user to connect as (default: $RHEV_USERNAME)')
    parser.add_option('-p', '--password', dest='passwd',
                      help='the API user\'s password (default: $RHEV_PASSWORD)')
    parser.add_option('-P', '--dwh-password', dest='dwhpasswd',
                      help='data warehouse password (default: same as --password)')
    parser.add_option('-d', '--debug', action='store_true',
                      help='enable debugging')
    parser.add_option('-v', '--verbose', action='store_const', const=10,
                      dest='verbosity', help='be verbose')
    parser.set_default('debug', False)
    parser.set_default('verbosity', 0)
    return parser


def parse_args():
    global opts, args
    opts, args = parser.parse_args()
    if opts.url is None:
        opts.url = os.environ.get('RHEV_URL')
    if opts.user is None:
        opts.user = os.environ.get('RHEV_USERNAME')
    if opts.passwd is None:
        opts.passwd = os.environ.get('RHEV_PASSWORD')
    if opts.url is None or opts.user is None or opts.passwd is None:
        sys.stderr.write('Error: incomplete connection parameters\n')
        sys.stderr.write('You must specify --url, --username and --password,\n')
        sys.stderr.write('or specify them with the $RHEV_URL, $RHEV_USERNAME\n')
        sys.stderr.write('and $RHEV_PASSWORD enviroment variables, respectively.\n')
        sys.exit(1)
    opts.dwhuser = 'rhevm'
    opts.dwhname = 'rhevm_history'
    netloc = urlparse(opts.url).netloc
    if ':' in netloc:
        netloc, port = netloc.split(':')
    opts.dwhhost = netloc
    if opts.dwhpasswd is None:
        opts.dwhpasswd = opts.passwd
    setup_logging(opts.debug)
    return opts, args


def print_help():
    parser.print_help()


def connect():
    conn = rhev.Connection(opts.url, opts.user, opts.passwd)
    conn.ping()
    conn.verbosity = opts.verbosity
    return conn


def connect_dwh():
    conn = psycopg2.connect(user=opts.dwhuser, password=opts.dwhpasswd,
                            host=opts.dwhhost, database=opts.dwhname)
    return conn


def excepthook(typ, value, trace):
    sys.stderr.write('Error: %s\n' % str(value))
    if opts.debug:
        fmt = traceback.format_exception(typ, value, trace)
        for line in fmt:
            sys.stderr.write(line)
        sys.stderr.write('\n')
    else:
        sys.stderr.write('Use --debug to get more information\n')
    sys.exit(1)

sys.excepthook = excepthook
