#!/usr/bin/env python
#
# This file is part of python-rhev. python-rhev is free software that is
# made available under the MIT license. Consult the file "LICENSE" that
# is distributed together with this file for the exact licensing terms.
#
# python-rhev is copyright (c) 2010-2011 by the python-rhev authors. See
# the file "AUTHORS" for a complete overview.

# This example shows how to use the events collection to keep an up to date
# view of the RHEV inventory, without having to periodically make a full dump.
# This example only synchronises a few attributes of virtual machines. The
# results are put in a SQLite database.

import sys
import os.path
import logging

import rhev
from rhev import schema
import common
import sqlite3

dbname = 'vms.sqlite'


def initialize():
    """Create the DB and perform the initial sync (dump)."""
    try:
        os.remove(dbname)
    except OSError:
        pass
    db = sqlite3.connect(dbname)
    cursor = db.cursor()
    cursor.execute("""
            CREATE TABLE vms
            (id TEXT, name TEXT, description TEXT,
             memory INTEGER, cores INTEGER)""")
    cursor.execute("""
            CREATE TABLE runs
            (date DATETIME, last_event INTEGER)
            """)
    event = api.get(schema.Event, search='sortby time desc')
    cursor.execute("""
            INSERT INTO runs
            VALUES (date('now'), ?)
            """, (event.id,))
    sys.stdout.write('last event id: %d\n' % int(event.id))
    vms = 0
    page = 1
    while True:
        query = 'sortby name asc page %d' % page
        batch = api.getall(schema.Event, search=query)
        if len(batch) < 10:
            break
        for vm in batch:
            cursor.execute("""
                    INSERT INTO vms
                    VALUES (?, ?, ?, ?, ?)
                    """, (vm.id, vm.name, vm.description, vm.memory,
                          vm.cpu.topology.sockets * vm.cpu.topology.cores))
        vms += len(batch)
        page += 1
    db.commit()
    sys.stdout.write('stored %d vm definitions into db\n' % vms)


def synchronize():
    """Perform one synchronization run."""
    db = sqlite3.connect(dbname)
    logger = logging.getLogger()
    cursor = db.cursor()
    cursor.execute('SELECT max(last_event) FROM runs')
    last = cursor.fetchone()[0]
    sys.stdout.write('starting at event: %d\n' % last)
    page = 1
    while True:
        query = 'sortby time asc page %d' % page
        batch = api.getall(schema.Event, search=query, from_=last)
        for event in batch:
            if event.code == 34:  # vm created
                vm = api.get(schema.VM, id=event.vm.id)
                if vm:
                    cursor.execute("""
                        INSERT INTO vms
                        VALUES (?, ?, ?, ?, ?)
                        """, (vm.id, vm.name, vm.description, vm.memory,
                              vm.cpu.topology.sockets * vm.cpu.topology.cores))
                    sys.stdout.write('detected new vm: %s\n' % vm.name)
            elif event.code == 35:  # vm updated
                vm = api.get(schema.VM, id=event.vm.id)
                if vm:
                    cursor.execute("""
                            UPDATE VMS
                            SET name = ?, description = ?, memory = ?,
                                cores = ?
                            WHERE id = ?
                            """, (vm.name, vm.description, vm.memory,
                                  vm.cpu.topology.sockets * vm.cpu.topology.cores,
                                  vm.id))
                    sys.stdout.write('detected updated vm: %s\n' % vm.name)
            elif event.code == 113:  # vm delete completed
                cursor.execute("""
                        DELETE FROM vms
                        WHERE id = ?
                        """, (event.vm.id,))
                sys.stdout.write('detected removed vm: %s\n' % event.vm.id)
            last = max(last, int(event.id))
        if len(batch) < 10:
            break
    cursor.execute("""
            INSERT INTO runs
            VALUES (date('now'), ?)
            """, (last,))
    sys.stdout.write('last event processed: %d\n' % last)
    db.commit()


common.create_parser(usage='%prog init\n       %prog sync',
                     description='sync virtual machines to local database')
opts, args = common.parse_args()
if len(args) != 1:
    common.print_help()
    sys.exit(1)

api = common.connect()

if len(args) != 1:
    common.print_help()
    sys.exit(1)

if sys.argv[1] == 'init':
    initialize()
elif sys.argv[1] == 'sync':
    synchronize()
else:
    common.print_help()
    sys.exit(1)
